@@ -1,12 +1,12 @@
 ##############################################################################
 ## package 'secrdesign'
 ## utility.R
-## 2022-10-23
+## 2022-10-23, 2022-12-04
 ##############################################################################
 
 .local <- new.env()
-.local$packageType <- "pre-release"
-#.local$packageType <- ""
+# .local$packageType <- "pre-release"
+.local$packageType <- ""
 .local$originCounter <- 1
 
 ##############################################################################
@@ -54,7 +54,7 @@ defaultmodel <- function (CL, detectfn) {
         model <- list(beta0 = ~ 1, beta1 = ~ 1)
     else ## detectfn %in% c(14:18))
         model <- list(lambda0 = ~ 1, sigma = ~ 1)
-    if (!CL) model <- c(list(D = ~1), model)
+    if (!is.null(CL) && !CL) model <- c(list(D = ~1), model)
     model
 }
 ##############################################################################
